/*
 *
 * dq par Thibaut Maquet <www.pagasa.net>
 *
 * Pour compiler, tapez simplement make
 */
 
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <syslog.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <getopt.h>

#define SMTP_TIMEOUT 15

#define INFO1 "Usage: dq serveur @domaine|#queue"
#define INFO2 "ETRN en cours."

#define ERR_TXT1 "Erreur: la machine est injoignable"
#define ERR_TXT2 "Erreur: le serveur ne rpond pas"
#define ERR_TXT3 "Erreur: mauvaise rponse SMTP"
#define ERR_TXT4 "Erreur: le serveur ne comprend pas le ESMTP"

#define SMTP1 "EHLO mon.serveur\n\r"
#define SMTP2 "ETRN %s\n\r"

/*
 * Protos
 */
 
int getconnect(char *,int);
int getdisconnect(int);
int readsock(int,char *,int);
int writesock(int,char *,int);
void usage(void);
int chksmtp(char *);
int dq(char *,char *);
int main(int, char **);


/*
 *Fonctions
 */
 
int getconnect(char *hostname,int port)
{
    int sk;
    struct sockaddr_in sa_in; 
    struct hostent *hs;   /* netdb.h */
    unsigned long ipadr;

    if ((ipadr=inet_addr(hostname))!=-1)
      sa_in.sin_addr.s_addr=ipadr;
    else
    {
      if ((hs=(struct hostent *)gethostbyname(hostname))==NULL)
        return -1; /* ERR_HOST */
      memset(&sa_in,0,sizeof(struct sockaddr_in));
      memcpy(&sa_in.sin_addr.s_addr,hs->h_addr,hs->h_length);
    }
    sa_in.sin_family=AF_INET;
    sa_in.sin_port=htons(port);
    if ((sk=socket(AF_INET, SOCK_STREAM,0))==-1)
      return -2; /* ERR_SOCKET*/
    if (connect(sk,(struct sockaddr *)&sa_in,sizeof(struct sockaddr_in))==-1)
      return -3; /* ERR_CONNECT */
    return sk;
}

int getdisconnect(int socket)
{
  if (socket)
    if (close(socket)==-1)
      return 0;
  return -1;
}

int readsock(int sk,char *buff,int wait)
{
  int n,leng;
  char *t=buff;
  fd_set rfds;
  struct timeval tv;
  struct sockaddr_in sa_in;
  
  FD_ZERO(&rfds);
  FD_SET(sk,&rfds);

  tv.tv_sec=wait;
  tv.tv_usec=0;
  
  if(!select(sk+1,&rfds,NULL,NULL,&tv))
    return 0;
  leng=sizeof(sa_in); 
  n=recvfrom(sk,buff,512,0,(struct sockaddr *)&sa_in,&leng);
                                                   
  t+=n;  
  *t=0;
  return n;
}

int writesock(int sk,char *buff,int len)
{
  if (send(sk,buff,len,0)==-1)
    return 0;
  return -1;
}

void usage(void)
{
  puts(INFO1);
}

int chksmtp(char *buff)
{
  int rfc;
  char rep[5];
  
  memcpy(rep,buff,4);
  rep[3]=0;
  rfc=atoi(rep);

  if(rfc<400)
    return 1;
  else
    return 0;
}        
    
int dq(char *h,char *d)
{
  int sk;
  char buff[512],err[80];
  
  memset(err,0,80);
  sk=getconnect(h,25);
  if(sk>0)
  {
    if(readsock(sk,buff,SMTP_TIMEOUT))  /* 220 Sendmail */
    {
      if(chksmtp(buff))
      {
        strcpy(buff,SMTP1);
        writesock(sk,buff,strlen(buff)); /* EHLO */
        if(readsock(sk,buff,SMTP_TIMEOUT)) /* 220 Bonjour */
        {
          if(chksmtp(buff))
          {
            sprintf(buff,SMTP2,d);
            writesock(sk,buff,strlen(buff)); /* ETRN */
            if(readsock(sk,buff,SMTP_TIMEOUT)) /* 220  ETRN lanc */
            {
              if(chksmtp(buff))
                puts(INFO2);
              else
                strcpy(err,ERR_TXT3);      
            }  
            else
              strcpy(err,ERR_TXT2);      
          }
          else
            strcpy(err,ERR_TXT4);      
        }
        else
          strcpy(err,ERR_TXT2);      
      }
      else
        strcpy(err,ERR_TXT3);      
    }
    else
      strcpy(err,ERR_TXT2);
    getdisconnect(sk);
  }
  else
    strcpy(err,ERR_TXT1);
  
  if(err)
    puts(err);  
  return 0;
}

int main(int argc, char **argv)
{
  int r;
  while((r=getopt(argc,argv,"h"))!=-1) 
  {
    switch(r)
    {
      case 'h':
        usage();
        return 1;
    }
  }
  
  if(argc!=3)
  {
    usage();
    return 1;
  }
  
  dq(argv[1],argv[2]);
  
  return 1;
}

